@@ -70,6 +70,14 @@
  * setting a new cdda://n+1 URI on playbin (as setting a new URI on playbin
  * involves closing and re-opening the CD device, which is much much slower).
  * </para>
+ * <title>Tags and meta-information</title>
+ * <para>
+ * CDDA sources will automatically emit a number of tags, details about which
+ * can be found in the libgsttag documentation. Those tags are:
+ * #GST_TAG_CDDA_CDDB_DISCID, #GST_TAG_CDDA_CDDB_DISCID_FULL,
+ * #GST_TAG_CDDA_MUSICBRAINZ_DISCID, #GST_TAG_CDDA_MUSICBRAINZ_DISCID_FULL,
+ * among others.
+ * </para>
  * </refsect2>
  */
 
@@ -182,23 +190,8 @@ gst_cdda_base_src_base_init (gpointer g_class)
   track_format = gst_format_register ("track", "CD track");
   sector_format = gst_format_register ("sector", "CD sector");
 
-  /* tags */
-  gst_tag_register (GST_TAG_CDDA_CDDB_DISCID, GST_TAG_FLAG_META,
-      G_TYPE_STRING, "discid", "CDDB discid for metadata retrieval",
-      gst_tag_merge_use_first);
-
-  gst_tag_register (GST_TAG_CDDA_CDDB_DISCID_FULL, GST_TAG_FLAG_META,
-      G_TYPE_STRING, "discid full",
-      "CDDB discid for metadata retrieval (full)", gst_tag_merge_use_first);
-
-  gst_tag_register (GST_TAG_CDDA_MUSICBRAINZ_DISCID, GST_TAG_FLAG_META,
-      G_TYPE_STRING, "musicbrainz-discid",
-      "Musicbrainz discid for metadata retrieval", gst_tag_merge_use_first);
-
-  gst_tag_register (GST_TAG_CDDA_MUSICBRAINZ_DISCID_FULL, GST_TAG_FLAG_META,
-      G_TYPE_STRING, "musicbrainz-discid-full",
-      "Musicbrainz discid for metadata retrieval (full)",
-      gst_tag_merge_use_first);
+  /* register CDDA tags */
+  gst_tag_register_musicbrainz_tags ();
 
 #if 0
   ///// FIXME: what type to use here? ///////
@@ -711,7 +704,7 @@ gst_cdda_base_src_do_seek (GstBaseSrc * basesrc, GstSegment * segment)
       seek_sector += src->tracks[0].start;
       break;
     default:
-      g_assert_not_reached ();
+      g_return_val_if_reached (FALSE);
   }
 
   src->cur_sector = (gint) seek_sector;
@@ -761,7 +754,7 @@ gst_cdda_base_src_handle_track_seek (GstCddaBaseSrc * src, gdouble rate,
         start_time = -1;
         break;
       default:
-        g_assert_not_reached ();
+        g_return_val_if_reached (FALSE);
     }
 
     switch (stop_type) {
@@ -786,7 +779,7 @@ gst_cdda_base_src_handle_track_seek (GstCddaBaseSrc * src, gdouble rate,
         stop_time = -1;
         break;
       default:
-        g_assert_not_reached ();
+        g_return_val_if_reached (FALSE);
     }
 
     GST_LOG_OBJECT (src, "seek segment %" GST_TIME_FORMAT "-%" GST_TIME_FORMAT,
@@ -817,7 +810,7 @@ gst_cdda_base_src_handle_track_seek (GstCddaBaseSrc * src, gdouble rate,
     return FALSE;
   }
 
-  GST_DEBUG_OBJECT (src, "seeking to track %d", start + 1);
+  GST_DEBUG_OBJECT (src, "seeking to track %" G_GINT64_FORMAT, start + 1);
 
   src->cur_sector = src->tracks[start].start;
   GST_DEBUG_OBJECT (src, "starting at sector %d", src->cur_sector);
@@ -891,7 +884,7 @@ gst_cdda_base_src_handle_event (GstBaseSrc * basesrc, GstEvent * event)
   return ret;
 }
 
-static guint
+static GstURIType
 gst_cdda_base_src_uri_get_type (void)
 {
   return GST_URI_SRC;
@@ -1032,7 +1025,10 @@ gst_cdda_base_src_add_track (GstCddaBaseSrc * src, GstCddaBaseSrcTrack * track)
   if (src->num_tracks > 0) {
     guint end_of_previous_track = src->tracks[src->num_tracks - 1].end;
 
-    g_return_val_if_fail (track->start >= end_of_previous_track, FALSE);
+    if (track->start < end_of_previous_track) {
+      GST_WARNING ("track %2u overlaps with previous tracks", track->num);
+      return FALSE;
+    }
   }
 
   GST_OBJECT_LOCK (src);
@@ -1488,6 +1484,7 @@ gst_cdda_base_src_stop (GstBaseSrc * basesrc)
   return TRUE;
 }
 
+
 static GstFlowReturn
 gst_cdda_base_src_create (GstPushSrc * pushsrc, GstBuffer ** buffer)
 {
@@ -1496,8 +1493,10 @@ gst_cdda_base_src_create (GstPushSrc * pushsrc, GstBuffer ** buffer)
   GstBuffer *buf;
   GstFormat format;
   gboolean eos;
-  gint64 position = GST_CLOCK_TIME_NONE;
-  gint64 duration = GST_CLOCK_TIME_NONE;
+
+  GstClockTime position = GST_CLOCK_TIME_NONE;
+  GstClockTime duration = GST_CLOCK_TIME_NONE;
+  gint64 qry_position;
 
   g_assert (klass->read_sector != NULL);
 
@@ -1511,7 +1510,7 @@ gst_cdda_base_src_create (GstPushSrc * pushsrc, GstBuffer ** buffer)
           src->cur_sector);
       break;
     default:
-      g_assert_not_reached ();
+      g_return_val_if_reached (GST_FLOW_ERROR);
   }
 
   if (eos) {
@@ -1551,12 +1550,14 @@ gst_cdda_base_src_create (GstPushSrc * pushsrc, GstBuffer ** buffer)
   }
 
   format = GST_FORMAT_TIME;
-  if (gst_pad_query_position (GST_BASE_SRC_PAD (src), &format, &position)) {
+  if (gst_pad_query_position (GST_BASE_SRC_PAD (src), &format, &qry_position)) {
     gint64 next_ts = 0;
 
+    position = (GstClockTime) qry_position;
+
     ++src->cur_sector;
     if (gst_pad_query_position (GST_BASE_SRC_PAD (src), &format, &next_ts)) {
-      duration = next_ts - position;
+      duration = (GstClockTime) (next_ts - qry_position);
     }
     --src->cur_sector;
   }
