@@ -25,6 +25,7 @@
 
 #include <string.h>
 #include <gst/gstutils.h>
+#include <gst/tag/tag.h>
 
 #include "riff-read.h"
 
@@ -89,7 +90,7 @@ skip_junk:
     goto too_small;
 
   *_chunk_data = buf;
-  *_offset += 8 + ((size + 1) & ~1);
+  *_offset += 8 + GST_ROUND_UP_2 (size);
 
   return GST_FLOW_OK;
 
@@ -165,7 +166,7 @@ gst_riff_parse_chunk (GstElement * element, GstBuffer * buf,
     *chunk_data = NULL;
 
   *_fourcc = fourcc;
-  *_offset += 8 + ((size + 1) & ~1);
+  *_offset += 8 + GST_ROUND_UP_2 (size);
 
   return TRUE;
 
@@ -470,8 +471,8 @@ gst_riff_parse_strf_auds (GstElement * element,
     len = GST_READ_UINT16_LE (&GST_BUFFER_DATA (buf)[16]);
     if (len + 2 + sizeof (gst_riff_strf_auds) > bufsize) {
       GST_WARNING_OBJECT (element,
-          "Extradata indicated %d bytes, but only %d available",
-          len, bufsize - 2 - sizeof (gst_riff_strf_auds));
+          "Extradata indicated %d bytes, but only %" G_GSSIZE_FORMAT
+          " available", len, bufsize - 2 - sizeof (gst_riff_strf_auds));
       len = bufsize - 2 - sizeof (gst_riff_strf_auds);
     }
     if (len)
@@ -499,7 +500,7 @@ gst_riff_parse_strf_auds (GstElement * element,
 too_small:
   {
     GST_ERROR_OBJECT (element,
-        "Too small strf_auds (%d available, %d needed)",
+        "Too small strf_auds (%d available, %" G_GSSIZE_FORMAT " needed)",
         bufsize, sizeof (gst_riff_strf_auds));
     gst_buffer_unref (buf);
     return FALSE;
@@ -570,7 +571,7 @@ gst_riff_parse_strf_iavs (GstElement * element,
 too_small:
   {
     GST_ERROR_OBJECT (element,
-        "Too small strf_iavs (%d available, %d needed)",
+        "Too small strf_iavs (%d available, %" G_GSSIZE_FORMAT " needed)",
         GST_BUFFER_SIZE (buf), sizeof (gst_riff_strf_iavs));
     gst_buffer_unref (buf);
     return FALSE;
@@ -595,9 +596,7 @@ gst_riff_parse_info (GstElement * element,
   guint size, tsize;
   guint32 tag;
   const gchar *type;
-  gchar *name;
   GstTagList *taglist;
-  gboolean have_tags = FALSE;
 
   g_return_if_fail (_taglist != NULL);
   g_return_if_fail (buf != NULL);
@@ -700,17 +699,19 @@ gst_riff_parse_info (GstElement * element,
         break;
     }
 
-    if (type) {
-      if (data[0] != '\0') {
-        /* read, NULL-terminate */
-        name = g_new (gchar, tsize + 1);
-        name[tsize] = '\0';
-        memcpy (name, data, tsize);
+    if (type != NULL && data[0] != '\0') {
+      static const gchar *env_vars[] = { "GST_AVI_TAG_ENCODING",
+        "GST_RIFF_TAG_ENCODING", "GST_TAG_ENCODING", NULL
+      };
+      gchar *val;
 
-        /* add to list */
-        have_tags = TRUE;
-        gst_tag_list_add (taglist, GST_TAG_MERGE_APPEND, type, name, NULL);
-        g_free (name);
+      val = gst_tag_freeform_string_to_utf8 ((gchar *) data, tsize, env_vars);
+
+      if (val) {
+        gst_tag_list_add (taglist, GST_TAG_MERGE_APPEND, type, val, NULL);
+        g_free (val);
+      } else {
+        GST_WARNING_OBJECT (element, "could not extract %s tag", type);
       }
     }
 
@@ -718,7 +719,7 @@ gst_riff_parse_info (GstElement * element,
     size -= tsize;
   }
 
-  if (have_tags) {
+  if (!gst_tag_list_is_empty (taglist)) {
     *_taglist = taglist;
   } else {
     *_taglist = NULL;
