@@ -197,7 +197,9 @@ static const GstTagEntryMatch tag_matches[] = {
   {GST_TAG_ALBUM, "TALB"},
   {GST_TAG_TRACK_NUMBER, "TRCK"},
   {GST_TAG_ARTIST, "TPE1"},
+  {GST_TAG_COMPOSER, "TCOM"},
   {GST_TAG_COPYRIGHT, "TCOP"},
+  {GST_TAG_COPYRIGHT_URI, "WCOP"},
   {GST_TAG_GENRE, "TCON"},
   {GST_TAG_DATE, "TDRC"},
   {GST_TAG_COMMENT, "COMM"},
@@ -207,6 +209,9 @@ static const GstTagEntryMatch tag_matches[] = {
   {GST_TAG_IMAGE, "APIC"},
   {GST_TAG_ENCODER, "TSSE"},
   {GST_TAG_BEATS_PER_MINUTE, "TBPM"},
+  {GST_TAG_ARTIST_SORTNAME, "TSOP"},
+  {GST_TAG_ALBUM_SORTNAME, "TSOA"},
+  {GST_TAG_TITLE_SORTNAME, "TSOT"},
   {NULL, NULL}
 };
 
@@ -244,18 +249,28 @@ static const GstTagEntryMatch user_tag_matches[] = {
   {GST_TAG_MUSICBRAINZ_ALBUMID, "TXXX|musicbrainz_albumid"},
   {GST_TAG_MUSICBRAINZ_ALBUMARTISTID, "TXXX|musicbrainz_albumartistid"},
   {GST_TAG_MUSICBRAINZ_TRMID, "TXXX|musicbrainz_trmid"},
+  {GST_TAG_CDDA_MUSICBRAINZ_DISCID, "TXXX|musicbrainz_discid"},
   /* musicbrainz identifiers according to spec no one pays
    * attention to (http://musicbrainz.org/docs/specs/metadata_tags.html) */
   {GST_TAG_MUSICBRAINZ_ARTISTID, "TXXX|MusicBrainz Artist Id"},
   {GST_TAG_MUSICBRAINZ_ALBUMID, "TXXX|MusicBrainz Album Id"},
   {GST_TAG_MUSICBRAINZ_ALBUMARTISTID, "TXXX|MusicBrainz Album Artist Id"},
   {GST_TAG_MUSICBRAINZ_TRMID, "TXXX|MusicBrainz TRM Id"},
+  /* according to: http://wiki.musicbrainz.org/MusicBrainzTag (yes, no space
+   * before 'ID' and not 'Id' either this time, yay for consistency) */
+  {GST_TAG_CDDA_MUSICBRAINZ_DISCID, "TXXX|MusicBrainz DiscID"},
   /* foobar2000 uses these identifiers to store gain/peak information in
    * ID3v2 tags <= v2.3.0. In v2.4.0 there's the RVA2 frame for that */
   {GST_TAG_TRACK_GAIN, "TXXX|replaygain_track_gain"},
   {GST_TAG_TRACK_PEAK, "TXXX|replaygain_track_peak"},
   {GST_TAG_ALBUM_GAIN, "TXXX|replaygain_album_gain"},
-  {GST_TAG_ALBUM_PEAK, "TXXX|replaygain_album_peak"}
+  {GST_TAG_ALBUM_PEAK, "TXXX|replaygain_album_peak"},
+  /* the following two are more or less made up, there seems to be little
+   * evidence that any popular application is actually putting this info
+   * into TXXX frames; the first one comes from a musicbrainz wiki 'proposed
+   * tags' page, the second one is analogue to the vorbis/ape/flac tag. */
+  {GST_TAG_CDDA_CDDB_DISCID, "TXXX|discid"},
+  {GST_TAG_CDDA_CDDB_DISCID, "TXXX|CDDB DiscID"}
 };
 
 /**
@@ -278,7 +293,8 @@ gst_tag_from_id3_user_tag (const gchar * type, const gchar * id3_user_tag)
 
   for (i = 0; i < G_N_ELEMENTS (user_tag_matches); ++i) {
     if (strncmp (type, user_tag_matches[i].original_tag, 4) == 0 &&
-        strcmp (id3_user_tag, user_tag_matches[i].original_tag + 5) == 0) {
+        g_ascii_strcasecmp (id3_user_tag,
+            user_tag_matches[i].original_tag + 5) == 0) {
       GST_LOG ("Mapped ID3v2 user tag '%s' to GStreamer tag '%s'",
           user_tag_matches[i].original_tag, user_tag_matches[i].gstreamer_tag);
       return user_tag_matches[i].gstreamer_tag;
@@ -319,66 +335,14 @@ static void
 gst_tag_extract_id3v1_string (GstTagList * list, const gchar * tag,
     const gchar * start, const guint size)
 {
-  const gchar *env;
-  gsize bytes_read;
+  const gchar *env_vars[] = { "GST_ID3V1_TAG_ENCODING",
+    "GST_ID3_TAG_ENCODING", "GST_TAG_ENCODING", NULL
+  };
   gchar *utf8;
 
-  /* Should we try the charsets specified
-   * via environment variables FIRST ? */
-  if (g_utf8_validate (start, size, NULL)) {
-    utf8 = g_strndup (start, size);
-    goto beach;
-  }
+  utf8 = gst_tag_freeform_string_to_utf8 (start, size, env_vars);
 
-  env = g_getenv ("GST_ID3V1_TAG_ENCODING");
-  if (!env || *env == '\0')
-    env = g_getenv ("GST_ID3_TAG_ENCODING");
-  if (!env || *env == '\0')
-    env = g_getenv ("GST_TAG_ENCODING");
-
-  /* Try charsets specified via the environment */
-  if (env && *env != '\0') {
-    gchar **c, **csets;
-
-    csets = g_strsplit (env, G_SEARCHPATH_SEPARATOR_S, -1);
-
-    for (c = csets; c && *c; ++c) {
-      if ((utf8 =
-              g_convert (start, size, "UTF-8", *c, &bytes_read, NULL, NULL))) {
-        if (bytes_read == size) {
-          g_strfreev (csets);
-          goto beach;
-        }
-        g_free (utf8);
-        utf8 = NULL;
-      }
-    }
-  }
-  /* Try current locale (if not UTF-8) */
-  if (!g_get_charset (&env)) {
-    if ((utf8 = g_locale_to_utf8 (start, size, &bytes_read, NULL, NULL))) {
-      if (bytes_read == size) {
-        goto beach;
-      }
-      g_free (utf8);
-      utf8 = NULL;
-    }
-  }
-
-  /* Try ISO-8859-1 */
-  utf8 =
-      g_convert (start, size, "UTF-8", "ISO-8859-1", &bytes_read, NULL, NULL);
-  if (utf8 != NULL && bytes_read == size) {
-    goto beach;
-  }
-
-  g_free (utf8);
-  return;
-
-beach:
-
-  g_strchomp (utf8);
-  if (utf8 && utf8[0] != '\0') {
+  if (utf8 && *utf8 != '\0') {
     gst_tag_list_add (list, GST_TAG_MERGE_REPLACE, tag, utf8, NULL);
   }
 
