@@ -24,8 +24,10 @@
  * SECTION:element-adder
  *
  * <refsect2>
+ * <para>
  * The Adder allows to mix several streams into one by adding the data.
  * Mixed data is clamped to the min/max values of the data format.
+ * </para>
  * <title>Example launch line</title>
  * <para>
  * <programlisting>
@@ -159,6 +161,41 @@ MAKE_FUNC_NC (add_float64, gdouble, gdouble)
 MAKE_FUNC_NC (add_float32, gfloat, gfloat)
 /* *INDENT-ON* */
 
+/* we can only accept caps that we and downstream can handle. */
+static GstCaps *
+gst_adder_sink_getcaps (GstPad * pad)
+{
+  GstAdder *adder;
+  GstCaps *result, *peercaps, *sinkcaps;
+
+  adder = GST_ADDER (GST_PAD_PARENT (pad));
+
+  GST_OBJECT_LOCK (adder);
+  /* get the downstream possible caps */
+  peercaps = gst_pad_peer_get_caps (adder->srcpad);
+  /* get the allowed caps on this sinkpad, we use the fixed caps function so
+   * that it does not call recursively in this function. */
+  sinkcaps = gst_pad_get_fixed_caps_func (pad);
+  if (peercaps) {
+    /* if the peer has caps, intersect */
+    GST_DEBUG_OBJECT (adder, "intersecting peer and template caps");
+    result = gst_caps_intersect (peercaps, sinkcaps);
+    gst_caps_unref (peercaps);
+    gst_caps_unref (sinkcaps);
+  } else {
+    /* the peer has no caps (or there is no peer), just use the allowed caps
+     * of this sinkpad. */
+    GST_DEBUG_OBJECT (adder, "no peer caps, using sinkcaps");
+    result = sinkcaps;
+  }
+  GST_OBJECT_UNLOCK (adder);
+
+  return result;
+}
+
+/* the first caps we receive on any of the sinkpads will define the caps for all
+ * the other sinkpads because we can only mix streams with the same caps.
+ * */
 static gboolean
 gst_adder_setcaps (GstPad * pad, GstCaps * caps)
 {
@@ -169,6 +206,9 @@ gst_adder_setcaps (GstPad * pad, GstCaps * caps)
 
   adder = GST_ADDER (GST_PAD_PARENT (pad));
 
+  GST_LOG_OBJECT (adder, "setting caps on pad %p,%s to %" GST_PTR_FORMAT, pad,
+      GST_PAD_NAME (pad), caps);
+
   /* FIXME, see if the other pads can accept the format. Also lock the
    * format on the other pads to this new format. */
   GST_OBJECT_LOCK (adder);
@@ -248,7 +288,7 @@ not_supported:
 }
 
 /* FIXME, the duration query should reflect how long you will produce
- * data, that is the amount of stream time until you will emit EOS. 
+ * data, that is the amount of stream time until you will emit EOS.
  *
  * For synchronized mixing this is always the max of all the durations
  * of upstream since we emit EOS when all of them finished.
@@ -261,7 +301,7 @@ not_supported:
  * also become invalid again and we need to post a new DURATION
  * message to notify this fact to the parent.
  * For now we take the max of all the upstream elements so the simple
- * cases work at least somewhat. 
+ * cases work at least somewhat.
  */
 static gboolean
 gst_adder_query_duration (GstAdder * adder, GstQuery * query)
@@ -388,7 +428,7 @@ forward_event_func (GstPad * pad, GValue * ret, GstEvent * event)
   return TRUE;
 }
 
-/* forwards the event to all sinkpads, takes ownership of the 
+/* forwards the event to all sinkpads, takes ownership of the
  * event
  *
  * Returns: TRUE if the event could be forwarded on all
@@ -438,22 +478,21 @@ gst_adder_src_event (GstPad * pad, GstEvent * event)
       GstSeekType curtype;
       gint64 cur;
 
-      /* parse the flushing flag */
-      gst_event_parse_seek (event, NULL, NULL, &flags, &curtype, &cur, NULL,
-          NULL);
+      /* parse the seek parameters */
+      gst_event_parse_seek (event, &adder->segment_rate, NULL, &flags, &curtype,
+          &cur, NULL, NULL);
 
-      /* if we are not flushing, just forward */
-      if (!flags & GST_SEEK_FLAG_FLUSH)
-        goto done;
+      /* check if we are flushing */
+      if (flags & GST_SEEK_FLAG_FLUSH) {
+        /* make sure we accept nothing anymore and return WRONG_STATE */
+        gst_collect_pads_set_flushing (adder->collect, TRUE);
 
-      /* make sure we accept nothing anymore and return WRONG_STATE */
-      gst_collect_pads_set_flushing (adder->collect, TRUE);
+        /* flushing seek, start flush downstream, the flush will be done
+         * when all pads received a FLUSH_STOP. */
+        gst_pad_push_event (adder->srcpad, gst_event_new_flush_start ());
+      }
 
-      /* flushing seek, start flush downstream, the flush will be done
-       * when all pads received a FLUSH_STOP. */
-      gst_pad_push_event (adder->srcpad, gst_event_new_flush_start ());
-
-      /* now wait for the collected to be finished and mark a new 
+      /* now wait for the collected to be finished and mark a new
        * segment */
       GST_OBJECT_LOCK (adder->collect);
       if (curtype == GST_SEEK_TYPE_SET)
@@ -463,7 +502,6 @@ gst_adder_src_event (GstPad * pad, GstEvent * event)
       adder->segment_pending = TRUE;
       GST_OBJECT_UNLOCK (adder->collect);
 
-    done:
       result = forward_event (adder, event);
       break;
     }
@@ -495,7 +533,7 @@ gst_adder_sink_event (GstPad * pad, GstEvent * event)
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_FLUSH_STOP:
       /* mark a pending new segment. This event is synchronized
-       * with the streaming thread so we can safely update the 
+       * with the streaming thread so we can safely update the
        * variable without races. It's somewhat weird because we
        * assume the collectpads forwarded the FLUSH_STOP past us
        * and downstream (using our source pad, the bastard!).
@@ -600,7 +638,7 @@ gst_adder_request_new_pad (GstElement * element, GstPadTemplate * templ,
   g_free (name);
 
   gst_pad_set_getcaps_function (newpad,
-      GST_DEBUG_FUNCPTR (gst_pad_proxy_getcaps));
+      GST_DEBUG_FUNCPTR (gst_adder_sink_getcaps));
   gst_pad_set_setcaps_function (newpad, GST_DEBUG_FUNCPTR (gst_adder_setcaps));
   gst_collect_pads_add_pad (adder->collect, newpad, sizeof (GstCollectData));
 
@@ -663,6 +701,7 @@ gst_adder_collected (GstCollectPads * pads, gpointer user_data)
   GstBuffer *outbuf;
   GstFlowReturn ret;
   gpointer outbytes;
+  gboolean empty = TRUE;
 
   adder = GST_ADDER (user_data);
 
@@ -670,33 +709,38 @@ gst_adder_collected (GstCollectPads * pads, gpointer user_data)
   if (G_UNLIKELY (adder->func == NULL))
     goto not_negotiated;
 
-  outbuf = NULL;
-  outbytes = NULL;
-
   /* get available bytes for reading, this can be 0 which could mean
    * empty buffers or EOS, which we will catch when we loop over the
    * pads. */
   size = gst_collect_pads_available (pads);
 
   GST_LOG_OBJECT (adder,
-      "starting to cycle through channels, %d bytes available", size);
+      "starting to cycle through channels, %d bytes available (bps = %d)", size,
+      adder->bps);
+
+  outbuf = NULL;
+  outbytes = NULL;
 
   for (collected = pads->data; collected; collected = g_slist_next (collected)) {
     GstCollectData *data;
     guint8 *bytes;
     guint len;
+    GstBuffer *inbuf;
 
     data = (GstCollectData *) collected->data;
 
-    /* get pointer to copy size bytes */
-    len = gst_collect_pads_read (pads, data, &bytes, size);
-    /* length 0 means EOS or an empty buffer so we still need to flush in
+    /* get a subbuffer of size bytes */
+    inbuf = gst_collect_pads_take_buffer (pads, data, size);
+    /* NULL means EOS or an empty buffer so we still need to flush in
      * case of an empty buffer. */
-    if (len == 0) {
+    if (inbuf == NULL) {
       GST_LOG_OBJECT (adder, "channel %p: no bytes available", data);
       goto next;
     }
 
+    bytes = GST_BUFFER_DATA (inbuf);
+    len = GST_BUFFER_SIZE (inbuf);
+
     if (outbuf == NULL) {
       GST_LOG_OBJECT (adder, "channel %p: making output buffer of %d bytes",
           data, size);
@@ -707,36 +751,51 @@ gst_adder_collected (GstCollectPads * pads, gpointer user_data)
       outbytes = GST_BUFFER_DATA (outbuf);
       gst_buffer_set_caps (outbuf, GST_PAD_CAPS (adder->srcpad));
 
-      /* clear if we are only going to fill a partial buffer */
-      if (G_UNLIKELY (size > len))
+      if (!GST_BUFFER_FLAG_IS_SET (inbuf, GST_BUFFER_FLAG_GAP)) {
+        /* clear if we are only going to fill a partial buffer */
+        if (G_UNLIKELY (size > len))
+          memset (outbytes, 0, size);
+
+        GST_LOG_OBJECT (adder, "channel %p: copying %d bytes from data %p",
+            data, len, bytes);
+
+        /* and copy the data into it */
+        memcpy (outbytes, bytes, len);
+        empty = FALSE;
+      } else {
+        GST_LOG_OBJECT (adder, "channel %p: zeroing %d bytes from data %p",
+            data, len, bytes);
         memset (outbytes, 0, size);
-
-      GST_LOG_OBJECT (adder, "channel %p: copying %d bytes from data %p",
-          data, len, bytes);
-
-      /* and copy the data into it */
-      memcpy (outbytes, bytes, len);
+      }
     } else {
-      GST_LOG_OBJECT (adder, "channel %p: mixing %d bytes from data %p",
-          data, len, bytes);
-      /* other buffers, need to add them */
-      adder->func ((gpointer) outbytes, (gpointer) bytes, len);
+      if (!GST_BUFFER_FLAG_IS_SET (inbuf, GST_BUFFER_FLAG_GAP)) {
+        GST_LOG_OBJECT (adder, "channel %p: mixing %d bytes from data %p",
+            data, len, bytes);
+        /* other buffers, need to add them */
+        adder->func ((gpointer) outbytes, (gpointer) bytes, len);
+        empty = FALSE;
+      } else {
+        GST_LOG_OBJECT (adder, "channel %p: skipping %d bytes from data %p",
+            data, len, bytes);
+      }
     }
   next:
-    gst_collect_pads_flush (pads, data, len);
+    if (inbuf)
+      gst_buffer_unref (inbuf);
   }
 
   /* can only happen when no pads to collect or all EOS */
   if (outbuf == NULL)
     goto eos;
 
-  /* our timestamping is very simple, just an ever incrementing 
+  /* our timestamping is very simple, just an ever incrementing
    * counter, the new segment time will take care of their respective
    * stream time. */
   if (adder->segment_pending) {
     GstEvent *event;
 
     /* FIXME, use rate/applied_rate as set on all sinkpads.
+     * - currently we just set rate as received from last seek-event
      * We could potentially figure out the duration as well using
      * the current segment positions and the stated stop positions.
      * Also we just start from stream time 0 which is rather
@@ -748,7 +807,7 @@ gst_adder_collected (GstCollectPads * pads, gpointer user_data)
      * the later streams would be delayed until the stream times
      * match.
      */
-    event = gst_event_new_new_segment_full (FALSE, 1.0,
+    event = gst_event_new_new_segment_full (FALSE, adder->segment_rate,
         1.0, GST_FORMAT_TIME, adder->timestamp, -1, adder->segment_position);
 
     gst_pad_push_event (adder->srcpad, event);
@@ -770,6 +829,10 @@ gst_adder_collected (GstCollectPads * pads, gpointer user_data)
   GST_BUFFER_DURATION (outbuf) = adder->timestamp -
       GST_BUFFER_TIMESTAMP (outbuf);
 
+  /* if we only processed silence, mark output again as silence */
+  if (empty)
+    GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_GAP);
+
   /* send it out */
   GST_LOG_OBJECT (adder, "pushing outbuf, timestamp %" GST_TIME_FORMAT,
       GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)));
@@ -808,6 +871,7 @@ gst_adder_change_state (GstElement * element, GstStateChange transition)
       adder->offset = 0;
       adder->segment_pending = TRUE;
       adder->segment_position = 0;
+      adder->segment_rate = 1.0;
       gst_segment_init (&adder->segment, GST_FORMAT_UNDEFINED);
       gst_collect_pads_start (adder->collect);
       break;
