@@ -266,6 +266,28 @@ static PixFmtInfo pix_fmt_info[PIX_FMT_NB] = {
         /* .y_chroma_shift = */ 0,
         /* .depth          = */ 8,
       },
+  /* [PIX_FMT_ARGB32] = */ {
+        /* .format         = */ PIX_FMT_ARGB32,
+        /* .name           = */ "argb32",
+        /* .nb_channels    = */ 4,
+        /* .color_type     = */ FF_COLOR_RGB,
+        /* .pixel_type     = */ FF_PIXEL_PACKED,
+        /* .is_alpha       = */ 1,
+        /* .x_chroma_shift = */ 0,
+        /* .y_chroma_shift = */ 0,
+        /* .depth          = */ 8,
+      },
+  /* [PIX_FMT_ABGR32] = */ {
+        /* .format         = */ PIX_FMT_ABGR32,
+        /* .name           = */ "abgr32",
+        /* .nb_channels    = */ 4,
+        /* .color_type     = */ FF_COLOR_RGB,
+        /* .pixel_type     = */ FF_PIXEL_PACKED,
+        /* .is_alpha       = */ 1,
+        /* .x_chroma_shift = */ 0,
+        /* .y_chroma_shift = */ 0,
+        /* .depth          = */ 8,
+      },
   /* [PIX_FMT_RGB565] = */ {
         /* .format         = */ PIX_FMT_RGB565,
         /* .name           = */ "rgb565",
@@ -417,7 +439,8 @@ avcodec_get_pix_fmt (const char *name)
   return pix_fmt_info[i].format;
 }
 
-int
+#if 0
+static int
 avpicture_layout (const AVPicture * src, int pix_fmt, int width, int height,
     unsigned char *dest, int dest_size)
 {
@@ -468,6 +491,7 @@ avpicture_layout (const AVPicture * src, int pix_fmt, int width, int height,
 
   return size;
 }
+#endif
 
 int
 avpicture_get_size (int pix_fmt, int width, int height)
@@ -648,7 +672,7 @@ img_copy_plane (uint8_t * dst, int dst_wrap,
 /**
  * Copy image 'src' to 'dst'.
  */
-void
+static void
 img_copy (AVPicture * dst, const AVPicture * src,
     int pix_fmt, int width, int height)
 {
@@ -1796,6 +1820,68 @@ bitcopy_n (unsigned int a, int n)
 
 #include "imgconvert_template.h"
 
+/* argb32 handling */
+
+#define RGB_NAME argb32
+#define FMT_ARGB32
+
+#define RGB_IN(r, g, b, s)\
+{\
+    unsigned int v = ((const uint32_t *)(s))[0];\
+    r = (v >> 24) & 0xff;\
+    g = (v >> 16) & 0xff;\
+    b = (v >> 8) & 0xff;\
+}
+
+#define RGBA_IN(r, g, b, a, s)\
+{\
+    unsigned int v = ((const uint32_t *)(s))[0];\
+    r = (v >> 24) & 0xff;\
+    g = (v >> 16) & 0xff;\
+    b = (v >> 8) & 0xff;\
+    a = v & 0xff;\
+}
+
+#define RGBA_OUT(d, r, g, b, a)\
+{\
+    ((uint32_t *)(d))[0] = (r << 24) | (g << 16) | (b << 8) | a;\
+}
+
+#define BPP 4
+
+#include "imgconvert_template.h"
+
+/* abgr32 handling */
+
+#define RGB_NAME abgr32
+#define FMT_ABGR32
+
+#define RGB_IN(r, g, b, s)\
+{\
+    unsigned int v = ((const uint32_t *)(s))[0];\
+    r = v & 0xff;\
+    g = (v >> 8) & 0xff;\
+    b = (v >> 16) & 0xff;\
+}
+
+#define RGBA_IN(r, g, b, a, s)\
+{\
+    unsigned int v = ((const uint32_t *)(s))[0];\
+    r = v & 0xff;\
+    g = (v >> 8) & 0xff;\
+    b = (v >> 16) & 0xff;\
+    a = (v >> 24) & 0xff;\
+}
+
+#define RGBA_OUT(d, r, g, b, a)\
+{\
+    ((uint32_t *)(d))[0] = r | (g << 8) | (b << 16) | (a << 24 );\
+}
+
+#define BPP 4
+
+#include "imgconvert_template.h"
+
 static void
 mono_to_gray (AVPicture * dst, const AVPicture * src,
     int width, int height, int xor_mask)
@@ -1948,6 +2034,8 @@ static ConvertEntry convert_table[] = {
   {PIX_FMT_YUV420P, PIX_FMT_BGRx32, yuv420p_to_bgrx32},
   {PIX_FMT_YUV420P, PIX_FMT_RGBA32, yuv420p_to_rgba32},
   {PIX_FMT_YUV420P, PIX_FMT_BGRA32, yuv420p_to_bgra32},
+  {PIX_FMT_YUV420P, PIX_FMT_ARGB32, yuv420p_to_argb32},
+  {PIX_FMT_YUV420P, PIX_FMT_ABGR32, yuv420p_to_abgr32},
 
   {PIX_FMT_YUV422P, PIX_FMT_YUV422, yuv422p_to_yuv422},
   {PIX_FMT_YUV422P, PIX_FMT_UYVY422, yuv422p_to_uyvy422},
@@ -1964,6 +2052,8 @@ static ConvertEntry convert_table[] = {
   {PIX_FMT_YUVJ420P, PIX_FMT_BGR32, yuvj420p_to_bgrx32},
   {PIX_FMT_YUVJ420P, PIX_FMT_RGBA32, yuvj420p_to_rgba32},
   {PIX_FMT_YUVJ420P, PIX_FMT_BGRA32, yuvj420p_to_bgra32},
+  {PIX_FMT_YUVJ420P, PIX_FMT_ARGB32, yuvj420p_to_argb32},
+  {PIX_FMT_YUVJ420P, PIX_FMT_ABGR32, yuvj420p_to_abgr32},
 
   {PIX_FMT_YUVJ444P, PIX_FMT_RGB24, yuvj444p_to_rgb24},
 
@@ -1983,6 +2073,8 @@ static ConvertEntry convert_table[] = {
   {PIX_FMT_RGB24, PIX_FMT_RGBA32, rgb24_to_rgba32},
   {PIX_FMT_RGB24, PIX_FMT_BGR24, rgb24_to_bgr24},
   {PIX_FMT_RGB24, PIX_FMT_BGRA32, rgb24_to_bgra32},
+  {PIX_FMT_RGB24, PIX_FMT_ARGB32, rgb24_to_argb32},
+  {PIX_FMT_RGB24, PIX_FMT_ABGR32, rgb24_to_abgr32},
   {PIX_FMT_RGB24, PIX_FMT_GRAY8, rgb24_to_gray},
   {PIX_FMT_RGB24, PIX_FMT_PAL8, rgb24_to_pal8},
   {PIX_FMT_RGB24, PIX_FMT_YUV444P, rgb24_to_yuv444p},
@@ -2002,15 +2094,17 @@ static ConvertEntry convert_table[] = {
   {PIX_FMT_xRGB32, PIX_FMT_GRAY8, xrgb32_to_gray},
 
   {PIX_FMT_RGBA32, PIX_FMT_BGRA32, rgba32_to_bgra32},
+  {PIX_FMT_RGBA32, PIX_FMT_ABGR32, rgba32_to_abgr32},
+  {PIX_FMT_RGBA32, PIX_FMT_ARGB32, rgba32_to_argb32},
   {PIX_FMT_RGBA32, PIX_FMT_BGR32, rgba32_to_bgr32},
   {PIX_FMT_RGBA32, PIX_FMT_BGRx32, rgba32_to_bgrx32},
+  {PIX_FMT_RGBA32, PIX_FMT_ABGR32, rgba32_to_abgr32},
   {PIX_FMT_RGBA32, PIX_FMT_RGB24, rgba32_to_rgb24},
   {PIX_FMT_RGBA32, PIX_FMT_RGB555, rgba32_to_rgb555},
   {PIX_FMT_RGBA32, PIX_FMT_PAL8, rgba32_to_pal8},
   {PIX_FMT_RGBA32, PIX_FMT_YUV420P, rgba32_to_yuv420p},
   {PIX_FMT_RGBA32, PIX_FMT_GRAY8, rgba32_to_gray},
   {PIX_FMT_RGBA32, PIX_FMT_AYUV4444, rgba32_to_ayuv4444},
-  {PIX_FMT_BGRA32, PIX_FMT_AYUV4444, bgra32_to_ayuv4444},
 
   {PIX_FMT_BGR24, PIX_FMT_RGB24, bgr24_to_rgb24},
   {PIX_FMT_BGR24, PIX_FMT_YUV420P, bgr24_to_yuv420p},
@@ -2030,6 +2124,17 @@ static ConvertEntry convert_table[] = {
   {PIX_FMT_BGRA32, PIX_FMT_RGBA32, bgra32_to_rgba32},
   {PIX_FMT_BGRA32, PIX_FMT_YUV420P, bgra32_to_yuv420p},
   {PIX_FMT_BGRA32, PIX_FMT_GRAY8, bgra32_to_gray},
+  {PIX_FMT_BGRA32, PIX_FMT_AYUV4444, bgra32_to_ayuv4444},
+
+  {PIX_FMT_ABGR32, PIX_FMT_RGB24, abgr32_to_rgb24},
+  {PIX_FMT_ABGR32, PIX_FMT_RGBA32, abgr32_to_rgba32},
+  {PIX_FMT_ABGR32, PIX_FMT_YUV420P, abgr32_to_yuv420p},
+  {PIX_FMT_ABGR32, PIX_FMT_GRAY8, abgr32_to_gray},
+
+  {PIX_FMT_ARGB32, PIX_FMT_RGB24, argb32_to_rgb24},
+  {PIX_FMT_ARGB32, PIX_FMT_RGBA32, argb32_to_rgba32},
+  {PIX_FMT_ARGB32, PIX_FMT_YUV420P, argb32_to_yuv420p},
+  {PIX_FMT_ARGB32, PIX_FMT_GRAY8, argb32_to_gray},
 
   {PIX_FMT_RGB555, PIX_FMT_RGB24, rgb555_to_rgb24},
   {PIX_FMT_RGB555, PIX_FMT_RGB32, rgb555_to_rgba32},
@@ -2051,6 +2156,8 @@ static ConvertEntry convert_table[] = {
   {PIX_FMT_GRAY8, PIX_FMT_BGRx32, gray_to_bgrx32},
   {PIX_FMT_GRAY8, PIX_FMT_RGBA32, gray_to_rgba32},
   {PIX_FMT_GRAY8, PIX_FMT_BGRA32, gray_to_bgra32},
+  {PIX_FMT_GRAY8, PIX_FMT_ARGB32, gray_to_argb32},
+  {PIX_FMT_GRAY8, PIX_FMT_ABGR32, gray_to_abgr32},
   {PIX_FMT_GRAY8, PIX_FMT_MONOWHITE, gray_to_monowhite},
   {PIX_FMT_GRAY8, PIX_FMT_MONOBLACK, gray_to_monoblack},
 
@@ -2068,6 +2175,8 @@ static ConvertEntry convert_table[] = {
   {PIX_FMT_PAL8, PIX_FMT_BGRx32, pal8_to_bgrx32},
   {PIX_FMT_PAL8, PIX_FMT_RGBA32, pal8_to_rgba32},
   {PIX_FMT_PAL8, PIX_FMT_BGRA32, pal8_to_bgra32},
+  {PIX_FMT_PAL8, PIX_FMT_ARGB32, pal8_to_argb32},
+  {PIX_FMT_PAL8, PIX_FMT_ABGR32, pal8_to_abgr32},
 
   {PIX_FMT_UYVY411, PIX_FMT_YUV411P, uyvy411_to_yuv411p},
 
@@ -2091,7 +2200,7 @@ get_convert_table_entry (int src_pix_fmt, int dst_pix_fmt)
   return NULL;
 }
 
-int
+static int
 avpicture_alloc (AVPicture * picture, int pix_fmt, int width, int height)
 {
   unsigned int size;
@@ -2108,7 +2217,7 @@ fail:
   return -1;
 }
 
-void
+static void
 avpicture_free (AVPicture * picture)
 {
   av_free (picture->data[0]);
@@ -2418,6 +2527,12 @@ img_get_alpha_info (const AVPicture * src, int pix_fmt, int width, int height)
     case PIX_FMT_BGRA32:
       ret = get_alpha_info_bgra32 (src, width, height);
       break;
+    case PIX_FMT_ARGB32:
+      ret = get_alpha_info_argb32 (src, width, height);
+      break;
+    case PIX_FMT_ABGR32:
+      ret = get_alpha_info_abgr32 (src, width, height);
+      break;
     case PIX_FMT_RGB555:
       ret = get_alpha_info_rgb555 (src, width, height);
       break;
@@ -2480,6 +2595,7 @@ img_get_alpha_info (const AVPicture * src, int pix_fmt, int width, int height)
 #endif
 
 /* filter parameters: [-1 4 2 4 -1] // 8 */
+#if 0
 static void
 deinterlace_line (uint8_t * dst,
     const uint8_t * lum_m4, const uint8_t * lum_m3,
@@ -2571,10 +2687,12 @@ deinterlace_line_inplace (uint8_t * lum_m4, uint8_t * lum_m3, uint8_t * lum_m2,
   }
 #endif
 }
+#endif
 
 /* deinterlacing : 2 temporal taps, 3 spatial taps linear filter. The
    top field is copied as is, but the bottom field is deinterlaced
    against the top field. */
+#if 0
 static void
 deinterlace_bottom_field (uint8_t * dst, int dst_wrap,
     const uint8_t * src1, int src_wrap, int width, int height)
@@ -2630,10 +2748,11 @@ deinterlace_bottom_field_inplace (uint8_t * src1, int src_wrap,
   deinterlace_line_inplace (buf, src_m1, src_0, src_0, src_0, width);
   av_free (buf);
 }
-
+#endif
 
 /* deinterlace - if not supported return -1 */
-int
+#if 0
+static int
 avpicture_deinterlace (AVPicture * dst, const AVPicture * src,
     int pix_fmt, int width, int height)
 {
@@ -2676,5 +2795,6 @@ avpicture_deinterlace (AVPicture * dst, const AVPicture * src,
 #endif
   return 0;
 }
+#endif
 
 #undef FIX
