@@ -32,24 +32,6 @@
 #include <fcntl.h>
 
 
-/* control stuff stolen from fdsrc */
-#define CONTROL_STOP            'S'     /* stop the select call */
-#define CONTROL_SOCKETS(o)      o->control_fds
-#define WRITE_SOCKET(o)         o->control_fds[1]
-#define READ_SOCKET(o)          o->control_fds[0]
-
-#define SEND_COMMAND(o, command)          \
-G_STMT_START {                              \
-  unsigned char c; c = command;             \
-  write (WRITE_SOCKET(o), &c, 1);         \
-} G_STMT_END
-
-#define READ_COMMAND(o, command, res)        \
-G_STMT_START {                                 \
-  res = read(READ_SOCKET(o), &command, 1);   \
-} G_STMT_END
-
-
 GST_DEBUG_CATEGORY_STATIC (tcpclientsrc_debug);
 #define GST_CAT_DEFAULT tcpclientsrc_debug
 
@@ -150,13 +132,10 @@ gst_tcp_client_src_init (GstTCPClientSrc * this, GstTCPClientSrcClass * g_class)
 {
   this->port = TCP_DEFAULT_PORT;
   this->host = g_strdup (TCP_DEFAULT_HOST);
-  this->sock_fd = -1;
+  this->sock_fd.fd = -1;
   this->protocol = GST_TCP_PROTOCOL_NONE;
   this->caps = NULL;
 
-  READ_SOCKET (this) = -1;
-  WRITE_SOCKET (this) = -1;
-
   gst_base_src_set_live (GST_BASE_SRC (this), TRUE);
 
   GST_OBJECT_FLAG_UNSET (this, GST_TCP_CLIENT_SRC_OPEN);
@@ -168,6 +147,8 @@ gst_tcp_client_src_finalize (GObject * gobject)
   GstTCPClientSrc *this = GST_TCP_CLIENT_SRC (gobject);
 
   g_free (this->host);
+
+  G_OBJECT_CLASS (parent_class)->finalize (gobject);
 }
 
 static GstCaps *
@@ -205,8 +186,8 @@ gst_tcp_client_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
   /* read the buffer header if we're using a protocol */
   switch (src->protocol) {
     case GST_TCP_PROTOCOL_NONE:
-      ret = gst_tcp_read_buffer (GST_ELEMENT (src), src->sock_fd,
-          READ_SOCKET (src), outbuf);
+      ret = gst_tcp_read_buffer (GST_ELEMENT (src), src->sock_fd.fd,
+          src->fdset, outbuf);
       break;
 
     case GST_TCP_PROTOCOL_GDP:
@@ -215,8 +196,8 @@ gst_tcp_client_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
         GstCaps *caps;
 
         GST_DEBUG_OBJECT (src, "getting caps through GDP");
-        ret = gst_tcp_gdp_read_caps (GST_ELEMENT (src), src->sock_fd,
-            READ_SOCKET (src), &caps);
+        ret = gst_tcp_gdp_read_caps (GST_ELEMENT (src), src->sock_fd.fd,
+            src->fdset, &caps);
 
         if (ret != GST_FLOW_OK)
           goto no_caps;
@@ -225,8 +206,8 @@ gst_tcp_client_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
         src->caps = caps;
       }
 
-      ret = gst_tcp_gdp_read_buffer (GST_ELEMENT (src), src->sock_fd,
-          READ_SOCKET (src), outbuf);
+      ret = gst_tcp_gdp_read_buffer (GST_ELEMENT (src), src->sock_fd.fd,
+          src->fdset, outbuf);
       break;
     default:
       /* need to assert as buf == NULL */
@@ -321,22 +302,18 @@ gst_tcp_client_src_start (GstBaseSrc * bsrc)
   gchar *ip;
   GstTCPClientSrc *src = GST_TCP_CLIENT_SRC (bsrc);
 
-  /* create the control sockets before anything */
-  if (socketpair (PF_UNIX, SOCK_STREAM, 0, CONTROL_SOCKETS (src)) < 0)
+  if ((src->fdset = gst_poll_new (TRUE)) == NULL)
     goto socket_pair;
 
-  fcntl (READ_SOCKET (src), F_SETFL, O_NONBLOCK);
-  fcntl (WRITE_SOCKET (src), F_SETFL, O_NONBLOCK);
-
   /* create receiving client socket */
   GST_DEBUG_OBJECT (src, "opening receiving client socket to %s:%d",
       src->host, src->port);
 
-  if ((src->sock_fd = socket (AF_INET, SOCK_STREAM, 0)) == -1)
+  if ((src->sock_fd.fd = socket (AF_INET, SOCK_STREAM, 0)) == -1)
     goto no_socket;
 
   GST_DEBUG_OBJECT (src, "opened receiving client socket with fd %d",
-      src->sock_fd);
+      src->sock_fd.fd);
   GST_OBJECT_FLAG_SET (src, GST_TCP_CLIENT_SRC_OPEN);
 
   /* look up name if we need to */
@@ -353,7 +330,7 @@ gst_tcp_client_src_start (GstBaseSrc * bsrc)
   g_free (ip);
 
   GST_DEBUG_OBJECT (src, "connecting to server");
-  ret = connect (src->sock_fd, (struct sockaddr *) &src->server_sin,
+  ret = connect (src->sock_fd.fd, (struct sockaddr *) &src->server_sin,
       sizeof (src->server_sin));
 
   if (ret) {
@@ -404,10 +381,13 @@ gst_tcp_client_src_stop (GstBaseSrc * bsrc)
   src = GST_TCP_CLIENT_SRC (bsrc);
 
   GST_DEBUG_OBJECT (src, "closing socket");
-  if (src->sock_fd != -1) {
-    close (src->sock_fd);
-    src->sock_fd = -1;
+
+  if (src->fdset != NULL) {
+    gst_poll_free (src->fdset);
+    src->fdset = NULL;
   }
+
+  gst_tcp_socket_close (&src->sock_fd);
   src->caps_received = FALSE;
   if (src->caps) {
     gst_caps_unref (src->caps);
@@ -415,11 +395,6 @@ gst_tcp_client_src_stop (GstBaseSrc * bsrc)
   }
   GST_OBJECT_FLAG_UNSET (src, GST_TCP_CLIENT_SRC_OPEN);
 
-  close (READ_SOCKET (src));
-  close (WRITE_SOCKET (src));
-  READ_SOCKET (src) = -1;
-  WRITE_SOCKET (src) = -1;
-
   return TRUE;
 }
 
@@ -429,7 +404,7 @@ gst_tcp_client_src_unlock (GstBaseSrc * bsrc)
 {
   GstTCPClientSrc *src = GST_TCP_CLIENT_SRC (bsrc);
 
-  SEND_COMMAND (src, CONTROL_STOP);
+  gst_poll_set_flushing (src->fdset, TRUE);
 
   return TRUE;
 }
